local LHR = select(2, ...)

local AceComm = LibStub("AceComm-3.0")
local AceSerializer = LibStub("AceSerializer-3.0")

-- Register comm prefix at initialization steps
function LHR:initComms()

    LHR.syncVersion = 0
    LHR.syncLastSender = ''

    AceComm:RegisterComm(LHR.constants.commsPrefix, LHR.OnCommReceived)
end

-- Handle message reception and
function LHR.OnCommReceived(prefix, data, channel, sender)

    if not UnitIsUnit('player', sender) then

        local success, message = AceSerializer:Deserialize(data)

        if (success) then
            if (message.type == LHR.constants.commsTypes.healingDone) then
                LHR:receiveSyncHeal(prefix, message, channel, sender)
            elseif (message.type == LHR.constants.commsTypes.syncOrder) then
                LHR:receiveSyncOrder(prefix, message, channel, sender)
            elseif (message.type == LHR.constants.commsTypes.syncRequest) then
                LHR:receiveSyncRequest(prefix, message, channel, sender)
            elseif (message.type == LHR.constants.commsTypes.backupRequest) then
                LHR:receiveBackupRequest(prefix, message, channel, sender)
            end
        end
    end
end

-- Checks if a given version from a given sender should be applied
function LHR:isVersionEligible(version, sender)
    return version > LHR.syncVersion or (version == LHR.syncVersion and sender < LHR.syncLastSender)
end

-----------------------------------------------------------------------------------------------------------------------
-- Messaging functions
-----------------------------------------------------------------------------------------------------------------------

-- Proxy to send raid addon message
function LHR:sendRaidAddonMessage(message)
    LHR:sendAddonMessage(message, LHR.constants.commsChannel)
end

-- Proxy to send whisper addon message
function LHR:sendWhisperAddonMessage(message, name)
    LHR:sendAddonMessage(message, 'WHISPER', name)
end

-- Broadcast a given message to the commsChannel with the commsPrefix
function LHR:sendAddonMessage(message, channel, name)
    AceComm:SendCommMessage(
        LHR.constants.commsPrefix,
        AceSerializer:Serialize(message),
        channel,
        name
    )
end

-----------------------------------------------------------------------------------------------------------------------
-- OUTPUT
-----------------------------------------------------------------------------------------------------------------------

-- Broadcast a healing event
function LHR:sendSyncHeal(healer, timestamp)
    local message = {
        ['type'] = LHR.constants.commsTypes.healingDone,
        ['timestamp'] = timestamp,
        ['player'] = healer.name,
    }

    LHR:sendRaidAddonMessage(message)
end

-- Broadcast current rotation configuration
function LHR:sendSyncOrder(whisper, name)

    LHR.syncVersion = LHR.syncVersion + 1
    LHR.syncLastSender = UnitName("player")

    local message = {
        ['type'] = LHR.constants.commsTypes.syncOrder,
        ['version'] = LHR.syncVersion,
        ['rotation'] = LHR:getSimpleRotationTables()
    }

    if (whisper) then
        LHR:sendWhisperAddonMessage(message, name)
    else
        LHR:sendRaidAddonMessage(message, name)
    end
end

-- Broadcast a request for the current rotation configuration
function LHR:sendSyncOrderRequest()

    local message = {
        ['type'] = LHR.constants.commsTypes.syncRequest,
    }

    LHR:sendRaidAddonMessage(message)
end

-- Broadcast a request for the current rotation configuration
function LHR:sendBackupRequest(name)

    LHR:printPrefixedMessage('Sending backup request to ' .. name)

    local message = {
        ['type'] = LHR.constants.commsTypes.backupRequest,
    }

    LHR:sendWhisperAddonMessage(message, name)
end

-----------------------------------------------------------------------------------------------------------------------
-- INPUT
-----------------------------------------------------------------------------------------------------------------------

-- Healshot event received
function LHR:receiveSyncHeal(prefix, message, channel, sender)

    local healer = LHR:getHealer(message.player)

    if (healer ~= nil) then
        LHR:rotate(LHR:getHealer(message.player))
    end
end

-- Rotation configuration received
function LHR:receiveSyncOrder(prefix, message, channel, sender)

    LHR:updateRaidStatus()

    if (LHR:isVersionEligible(message.version, sender)) then
        LHR.syncVersion = (message.version)
        LHR.syncLastSender = sender

        LHR:printPrefixedMessage('Neue Rotation erhalten von ' .. sender)
        LHR:applyRotationConfiguration(message.rotation)
    end
end

-- Request to send current roration configuration received
function LHR:receiveSyncRequest(prefix, data, channel, sender)
    LHR:sendSyncOrder(true, sender)
end

-- Received a backup request
function LHR:receiveBackupRequest(prefix, message, channel, sender)
    LHR:printPrefixedMessage(sender .. ' bittet um backup !')
    LHR:throwHealAlert()
end
