local LHR = select(2, ...)

-- Create main window
function LHR:createMainFrame()
    LHR.mainFrame = CreateFrame("Frame", 'mainFrame', UIParent)
    LHR.mainFrame:SetWidth(LHR.constants.mainFrameWidth)
    LHR.mainFrame:SetHeight(LHR.constants.rotationFramesBaseHeight * 2 + LHR.constants.titleBarHeight)
    LHR.mainFrame:Show()

    LHR.mainFrame:RegisterForDrag("LeftButton")
    LHR.mainFrame:SetClampedToScreen(true)
    LHR.mainFrame:SetScript("OnDragStart", function() LHR.mainFrame:StartMoving() end)

    LHR.mainFrame:SetScript(
        "OnDragStop",
        function()
            local config = LHR.db.profile
            LHR.mainFrame:StopMovingOrSizing()

            config.point = 'TOPLEFT'
            config.y = LHR.mainFrame:GetTop()
            config.x = LHR.mainFrame:GetLeft()
        end
    )
end

-- Create Title frame
function LHR:createTitleFrame()
    LHR.mainFrame.titleFrame = CreateFrame("Frame", 'rotationFrame', LHR.mainFrame)
    LHR.mainFrame.titleFrame:SetPoint('TOPLEFT')
    LHR.mainFrame.titleFrame:SetPoint('TOPRIGHT')
    LHR.mainFrame.titleFrame:SetHeight(LHR.constants.titleBarHeight)

    LHR.mainFrame.titleFrame.texture = LHR.mainFrame.titleFrame:CreateTexture(nil, "BACKGROUND")
    LHR.mainFrame.titleFrame.texture:SetColorTexture(LHR.colors.blue:GetRGB())
    LHR.mainFrame.titleFrame.texture:SetAllPoints()

    LHR.mainFrame.titleFrame.text = LHR.mainFrame.titleFrame:CreateFontString(nil, "ARTWORK")
    LHR.mainFrame.titleFrame.text:SetFont("Fonts\\ARIALN.ttf", 12)
    LHR.mainFrame.titleFrame.text:SetShadowColor(0,0,0,0.5)
    LHR.mainFrame.titleFrame.text:SetShadowOffset(1,-1)
    LHR.mainFrame.titleFrame.text:SetPoint("LEFT",5,0)
    LHR.mainFrame.titleFrame.text:SetText('LHR')
    LHR.mainFrame.titleFrame.text:SetTextColor(1,1,1,1)
end

-- Create title bar buttons
function LHR:createButtons()

    local buttons = {
        {
            ['texture'] = 'Interface/Buttons/UI-Panel-MinimizeButton-Up',
            ['callback'] = LHR.toggleDisplay,
            ['textCoord'] = {0.18, 0.8, 0.2, 0.8}
        },
        {
            ['texture'] = 'Interface/GossipFrame/BinderGossipIcon',
            ['callback'] = LHR.openSettings
        },
        {
            ['texture'] = 'Interface/Buttons/UI-RefreshButton',
            ['callback'] = function()
                    LHR:updateRaidStatus()
                    LHR:resetRotation()
                    LHR:sendSyncOrderRequest()
                end
        },
        {
            ['texture'] = 'Interface/Buttons/UI-GuildButton-MOTD-Up',
            ['callback'] = LHR.printRotationSetup
        },
    }

    local position = 5

    for key, button in pairs(buttons) do
        LHR:createButton(position, button.texture, button.callback, button.textCoord)
        position = position + 13
    end
end

-- Create a single button in the title bar
function LHR:createButton(position, texture, callback, textCoord)

    local button = CreateFrame("Button", nil, LHR.mainFrame.titleFrame)
    button:SetPoint('RIGHT', -position, 0)
    button:SetWidth(10)
    button:SetHeight(10)

    local normal = button:CreateTexture()
    normal:SetTexture(texture)
    normal:SetAllPoints()
    button:SetNormalTexture(normal)

    local highlight = button:CreateTexture()
    highlight:SetTexture(texture)
    highlight:SetAllPoints()
    button:SetHighlightTexture(highlight)

    if (textCoord) then
        normal:SetTexCoord(unpack(textCoord))
        highlight:SetTexCoord(unpack(textCoord))
    end

    button:SetScript("OnClick", callback)
end

-- Create rotation frame
function LHR:createRotationFrame()
    LHR.mainFrame.rotationFrame = CreateFrame("Frame", 'rotationFrame', LHR.mainFrame)
    LHR.mainFrame.rotationFrame:SetPoint('LEFT')
    LHR.mainFrame.rotationFrame:SetPoint('RIGHT')
    LHR.mainFrame.rotationFrame:SetPoint('TOP', 0, -LHR.constants.titleBarHeight)
    LHR.mainFrame.rotationFrame:SetHeight(LHR.constants.rotationFramesBaseHeight)

    LHR.mainFrame.rotationFrame.texture = LHR.mainFrame.rotationFrame:CreateTexture(nil, "BACKGROUND")
    LHR.mainFrame.rotationFrame.texture:SetColorTexture(0,0,0,0.5)
    LHR.mainFrame.rotationFrame.texture:SetAllPoints()
end

-- Create backup frame
function LHR:createBackupFrame()
    -- Backup frame
    LHR.mainFrame.backupFrame = CreateFrame("Frame", 'backupFrame', LHR.mainFrame)
    LHR.mainFrame.backupFrame:SetPoint('TOPLEFT', LHR.mainFrame.rotationFrame, 'BOTTOMLEFT', 0, 0)
    LHR.mainFrame.backupFrame:SetPoint('TOPRIGHT', LHR.mainFrame.rotationFrame, 'BOTTOMRIGHT', 0, 0)
    LHR.mainFrame.backupFrame:SetHeight(LHR.constants.rotationFramesBaseHeight)

    -- Set Texture
    LHR.mainFrame.backupFrame.texture = LHR.mainFrame.backupFrame:CreateTexture(nil, "BACKGROUND")
    LHR.mainFrame.backupFrame.texture:SetColorTexture(0,0,0,0.5)
    LHR.mainFrame.backupFrame.texture:SetAllPoints()

    -- Visual separator
    LHR.mainFrame.backupFrame.texture = LHR.mainFrame.backupFrame:CreateTexture(nil, "BACKGROUND")
    LHR.mainFrame.backupFrame.texture:SetColorTexture(0.8,0.8,0.8,0.8)
    LHR.mainFrame.backupFrame.texture:SetHeight(1)
    LHR.mainFrame.backupFrame.texture:SetWidth(60)
    LHR.mainFrame.backupFrame.texture:SetPoint('TOP')
end

-- Create single healer frame
function LHR:createHealerFrame(healer, parentFrame)
    healer.frame = CreateFrame("Frame", nil, parentFrame)
    healer.frame:SetHeight(LHR.constants.healerFrameHeight)

    -- Set Texture
    healer.frame.texture = healer.frame:CreateTexture(nil, "ARTWORK")
    healer.frame.texture:SetTexture("Interface\\AddOns\\LoathebHeilerRotation\\textures\\steel.tga")
    healer.frame.texture:SetAllPoints()

    -- Set Text
    healer.frame.text = healer.frame:CreateFontString(nil, "ARTWORK")
    healer.frame.text:SetFont(LHR:getPlayerNameFont(), 12)
    healer.frame.text:SetPoint("LEFT",5,0)
    healer.frame.text:SetText(healer.name)

    LHR:createCooldownFrame(healer)
    LHR:configureHealerFrameDrag(healer)

    LHR:toggleHealerFrameDragging(healer, LHR:isPlayerAllowedToSortHealerList())
end

-- Create the cooldown frame
function LHR:createCooldownFrame(healer)

    -- Frame
    healer.frame.cooldownFrame = CreateFrame("Frame", nil, healer.frame)
    healer.frame.cooldownFrame:SetPoint('LEFT', 5, 0)
    healer.frame.cooldownFrame:SetPoint('RIGHT', -5, 0)
    healer.frame.cooldownFrame:SetPoint('TOP', 0, -17)
    healer.frame.cooldownFrame:SetHeight(3)

    -- background
    healer.frame.cooldownFrame.background = healer.frame.cooldownFrame:CreateTexture(nil, "ARTWORK")
    healer.frame.cooldownFrame.background:SetColorTexture(0,0,0,1)
    healer.frame.cooldownFrame.background:SetAllPoints()

    local statusBar = CreateFrame("StatusBar", nil, healer.frame.cooldownFrame)
    statusBar:SetAllPoints()
    statusBar:SetMinMaxValues(0,1)

    local statusBarTexture = statusBar:CreateTexture(nil, "OVERLAY");
    statusBarTexture:SetColorTexture(1, 0, 0);
    statusBar:SetStatusBarTexture(statusBarTexture);

    healer.frame.cooldownFrame.statusBar = statusBar

    healer.frame.cooldownFrame:SetScript(
        "OnUpdate",
        function(self, elapsed)
            self.statusBar:SetValue(GetTime())

            if (self.statusBar.expirationTime < GetTime()) then
                self:Hide()
            end
        end
    )

    healer.frame.cooldownFrame:Hide()
end
