local LHR = select(2, ...)

local L = LHR.L

-- Initialize GUI frames. Shouldn't be called more than once
function LHR:initGui()

    LHR:createMainFrame()
    LHR:createTitleFrame()
    LHR:createButtons()
    LHR:createRotationFrame()
    LHR:createBackupFrame()

    LHR:drawHealerFrames()
    LHR:createDropHintFrame()
    LHR:createRulerFrame()

    LHR:updateDisplay()
end

-- Show/Hide main window based on user settings
function LHR:updateDisplay()

    if (not LHR.db.profile.doNotShowWindowOnRaidJoin and LHR:isInPveRaid()) then
        LHR.mainFrame:Show()
    else
        if (LHR.db.profile.hideNotInRaid) then
            LHR.mainFrame:Hide()
        end
    end
end

-- render / re-render healer frames to reflect table changes.
function LHR:drawHealerFrames()

    -- Different height to reduce spacing between both groups
    LHR.mainFrame:SetHeight(LHR.constants.rotationFramesBaseHeight + LHR.constants.titleBarHeight)
    LHR.mainFrame.rotationFrame:SetHeight(LHR.constants.rotationFramesBaseHeight)

    LHR:drawList(LHR.rotationTables.rotation, LHR.mainFrame.rotationFrame)

    if (#LHR.rotationTables.backup > 0) then
        LHR.mainFrame:SetHeight(LHR.mainFrame:GetHeight() + LHR.constants.rotationFramesBaseHeight)
    end

    LHR.mainFrame.backupFrame:SetHeight(LHR.constants.rotationFramesBaseHeight)
    LHR:drawList(LHR.rotationTables.backup, LHR.mainFrame.backupFrame)

end

-- Handle the render of a single healer frames group
function LHR:drawList(healerList, parentFrame)

    local index = 1
    local healerFrameHeight = LHR.constants.healerFrameHeight
    local healerFrameSpacing = LHR.constants.healerFrameSpacing

    if (#healerList < 1 and parentFrame == LHR.mainFrame.backupFrame) then
        parentFrame:Hide()
    else
        parentFrame:Show()
    end

    for key, healer in pairs(healerList) do

        -- Using existing frame if possible
        if (healer.frame == nil) then
            LHR:createHealerFrame(healer, parentFrame)
        else
            healer.frame:SetParent(parentFrame)
        end

        healer.frame:ClearAllPoints()
        healer.frame:SetPoint('LEFT', 10, 0)
        healer.frame:SetPoint('RIGHT', -10, 0)

        -- Setting top margin
        local marginTop = 10 + (index - 1) * (healerFrameHeight + healerFrameSpacing)
        healer.frame:SetPoint('TOP', parentFrame, 'TOP', 0, -marginTop)

        -- Handling parent windows height increase
        if (index == 1) then
            parentFrame:SetHeight(parentFrame:GetHeight() + healerFrameHeight)
            LHR.mainFrame:SetHeight(LHR.mainFrame:GetHeight() + healerFrameHeight)
        else
            parentFrame:SetHeight(parentFrame:GetHeight() + healerFrameHeight + healerFrameSpacing)
            LHR.mainFrame:SetHeight(LHR.mainFrame:GetHeight() + healerFrameHeight + healerFrameSpacing)
        end

        -- SetColor
        setHealerFrameColor(healer)

        healer.frame:Show()
        healer.frame.healer = healer

        index = index + 1
    end
end

-- Hide the healer frame
function LHR:hideHealer(healer)
    if (healer.frame ~= nil) then
        healer.frame:Hide()
    end
end

-- Refresh a single healer frame
function LHR:refreshHealerFrame(healer)
    setHealerFrameColor(healer)
end

function LHR:getHealerClassColor(healer)

	local healerClass = select(2, UnitClass(healer.name))
	
	if(healerClass == "PRIEST") then
		return LHR.classColors.priest
	elseif(healerClass == "PALADIN") then
		return LHR.classColors.paladin
	elseif(healerClass == "DRUID") then
		return LHR.classColors.druid
	elseif(healerClass == "SHAMAN") then
		return LHR.classColors.shaman
	end
	
	return LHR.colors.green
end

function LHR:getHealerTextColor(healer)

	local healerClass = select(2, UnitClass(healer.name))
	
	if(healerClass == "PRIEST") then
		return LHR.colors.black
	end
	
	return LHR.colors.white
end

-- Set the healer frame color regarding it's status
function setHealerFrameColor(healer)

    --local color = LHR.colors.green
	local color = LHR:getHealerClassColor(healer)
	local textColor = LHR:getHealerTextColor(healer)
	
    if (not LHR:isHealerOnline(healer)) then
        color = LHR.colors.gray
		textColor = LHR.colors.white
    elseif (not LHR:isHealerAlive(healer)) then
        color = LHR.colors.red
		textColor = LHR.colors.white
    elseif (healer.nextHeal) then
        color = LHR.colors.nextcaster
		textColor = LHR.colors.white
    end

    healer.frame.texture:SetVertexColor(color:GetRGB())
	healer.frame.text:SetTextColor(textColor:GetRGB())
end

function LHR:startHealerCooldown(healer)
    healer.frame.cooldownFrame.statusBar:SetMinMaxValues(GetTime(), GetTime() + 60)
    healer.frame.cooldownFrame.statusBar.expirationTime = GetTime() + 60
    healer.frame.cooldownFrame:Show()
end

function LHR:stopHealerCooldown(healer)
    healer.frame.cooldownFrame.statusBar:SetMinMaxValues(GetTime(), GetTime())
    healer.frame.cooldownFrame.statusBar.expirationTime = GetTime()
    healer.frame.cooldownFrame:Hide()
end

-- Lock/Unlock the mainFrame position
function LHR:lock(lock)
    LHR.db.profile.lock = lock
    LHR:applySettings()

    if (lock) then
        LHR:printMessage(L['WINDOW_LOCKED'])
    else
        LHR:printMessage(L['WINDOW_UNLOCKED'])
    end
end
