local LHR = select(2, ...)

-- Check if a table contains the given element
function LHR:tableContains(table, element)

    for _, value in pairs(table) do
        if value == element then
            return true
        end
    end

    return false
end

-- Checks if a healer is alive
function LHR:isHealerAlive(healer)
    return not UnitIsDeadOrGhost(healer.name)
end

-- Checks if a healer is offline
function LHR:isHealerOnline(healer)
    return UnitIsConnected(healer.name)
end

-- Checks if a healer is online and alive
function LHR:isHealerAliveAndOnline(healer)
    return LHR:isHealerOnline(healer) and LHR:isHealerAlive(healer)
end

-- Checks if a healer healing is ready
function LHR:isHealerHealCooldownReady(healer)
    return healer.lastHealTime <= GetTime() - 60
end

-- Checks if a healer is eligible to heal next
function LHR:isEligibleForNextHeal(healer)

    local isCooldownShortEnough = healer.lastHealTime <= GetTime() - LHR.constants.minimumCooldownElapsedForEligibility

    return LHR:isHealerAliveAndOnline(healer) and isCooldownShortEnough
end

-- Checks if a healer is in a battleground
function LHR:isPlayerInBattleground()
    return UnitInBattleground('player') ~= nil
end

-- Checks if a healer is in a PvE raid
function LHR:isInPveRaid()
    return IsInRaid() and not LHR:isPlayerInBattleground()
end

function LHR:getPlayerNameFont()
    return "Fonts\\ARIALN.ttf"
end

function LHR:getIdFromGuid(guid)
    local type, _, _, _, _, mobId, _ = strsplit("-", guid or "")
    return type, tonumber(mobId)
end

-- Checks if the player is a healer
function LHR:isHealer(name)
	local className = select(2,UnitClass(name))
	return className == 'PRIEST' or className == 'PALADIN' or className == 'DRUID' or className == "SHAMAN"
end

-- Check if unit is promoted (raid assist or raid leader)
function LHR:isPlayerRaidAssist(name)

    if (LHR:isInPveRaid()) then

        local raidIndex = UnitInRaid(name)

        if (raidIndex) then
            local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(raidIndex)

            if (rank > 0) then
                return true
            end
        end
    end

    return false
end

-- Checks if condition to enable drag and drop are met
function LHR:isPlayerAllowedToSortHealerList()
    local playerName = UnitName("player")
    return LHR:isHealer(playerName) or LHR:isPlayerRaidAssist(playerName)
end
